@@ -4,8 +4,8 @@ mod text_plotter;
 use std::net::SocketAddr;
 use std::str::FromStr;
 use structopt::StructOpt;
-use terminal_plotter::TerminalPlotter;
-use text_plotter::StdoutTextPlotter;
+use terminal_plotter::TerminalMultiPlotter;
+use text_plotter::StdoutTextMultiPlotter;
 use tokio::net::{TcpListener, TcpStream};
 use tokio_stream::StreamExt;
 use tokio_util::codec::{Framed, LinesCodec};
@@ -16,7 +16,7 @@ impl FromStr for PlotterType {
     fn from_str(day: &str) -> Result<Self, Self::Err> {
         match day {
             "text" => Ok(PlotterType::Text),
-            "tui" => Ok(PlotterType::Plotter),
+            "tui" => Ok(PlotterType::Terminal),
             _ => Err("Invalid plotter type"),
         }
     }
@@ -25,7 +25,7 @@ impl FromStr for PlotterType {
 #[derive(Debug, Copy, Clone)]
 enum PlotterType {
     Text,
-    Plotter,
+    Terminal,
 }
 
 impl Default for PlotterType {
@@ -98,10 +98,10 @@ async fn main() -> Result<(), Box<dyn std::error::Error>> {
     }
 }
 
-fn make_plotter(plotter_type: PlotterType, opt: PlotterOpt) -> Box<dyn Plotter + Send> {
+fn make_multi_plotter(plotter_type: PlotterType, opt: PlotterOpt) -> Box<dyn MultiPlotter> {
     match plotter_type {
-        PlotterType::Text => Box::new(StdoutTextPlotter::new(opt)),
-        PlotterType::Plotter => Box::new(TerminalPlotter::new(opt)),
+        PlotterType::Text => Box::new(StdoutTextMultiPlotter::new(opt)),
+        PlotterType::Terminal => Box::new(TerminalMultiPlotter::new(opt)),
     }
 }
 
@@ -111,13 +111,14 @@ async fn launch_multiple_connections_server(
 ) -> Result<(), Box<dyn std::error::Error>> {
     let plotter_type = opt.view;
     let plotter_opt = opt.into();
+    let mut multi_plotter = make_multi_plotter(plotter_type, plotter_opt);
 
     loop {
         let (socket, _) = listener.accept().await?;
+        let tp = multi_plotter.spawn();
 
         tokio::spawn(async move {
             let server = Framed::new(socket, LinesCodec::new_with_max_length(1024));
-            let tp = make_plotter(plotter_type, plotter_opt);
             process_incoming_data(tp, server).await
         });
     }
@@ -129,11 +130,12 @@ async fn launch_single_connection_server(
 ) -> Result<(), Box<dyn std::error::Error>> {
     let plotter_type = opt.view;
     let plotter_opt = opt.into();
+    let mut multi_plotter = make_multi_plotter(plotter_type, plotter_opt);
 
     loop {
         let (socket, _) = listener.accept().await?;
         let server = Framed::new(socket, LinesCodec::new_with_max_length(1024));
-        let tp = make_plotter(plotter_type, plotter_opt);
+        let tp = multi_plotter.spawn();
         process_incoming_data(tp, server).await
     }
 }
@@ -150,8 +152,13 @@ async fn process_incoming_data(
 }
 
 trait Plotter {
+    fn update(&mut self, y: f64);
+}
+
+trait MultiPlotter {
     fn new(opt: PlotterOpt) -> Self
     where
         Self: Sized;
-    fn update(&mut self, y: f64);
+
+    fn spawn(&mut self) -> Box<dyn Plotter + Send>;
 }
