@@ -1,6 +1,6 @@
 use super::Plotter;
 use super::PlotterOpt;
-use super::Range;
+use crate::MultiPlotter;
 use plotters::prelude::*;
 use plotters::style::text_anchor::{HPos, VPos};
 use plotters_backend::{
@@ -8,79 +8,37 @@ use plotters_backend::{
 };
 use std::collections::vec_deque::VecDeque;
 use std::error::Error;
-use std::sync::{Arc, Mutex};
+use tokio::time::{Duration, Instant};
+
+type Sender = tokio::sync::mpsc::UnboundedSender<HistoryRecord>;
 
 // FIXME Is this valid?
 unsafe impl Send for TerminalPlotter {}
 
+#[derive(Default)]
+struct History(VecDeque<Record>);
+
+type Record = (Instant, f64);
+
+#[derive(Debug)]
+struct HistoryRecord {
+    history_id: usize,
+    record: Record,
+}
+
 pub struct TerminalPlotter {
-    width: usize,
-    range: Range,
-    drawing_area: Arc<Mutex<DrawingArea<TextDrawingBackend, plotters::coord::Shift>>>,
-    series: VecDeque<(f64, f64)>,
+    history_id: usize,
+    tx: Sender,
 }
 
 impl Plotter for TerminalPlotter {
-    fn new(opt: PlotterOpt) -> Self {
-        let backend = TextDrawingBackend {
-            state: vec![PixelState::Empty; 5000],
-            width: opt.width,
-        };
-        let drawing_area = backend.into_drawing_area();
-        let drawing_area = Arc::new(Mutex::new(drawing_area));
-        let series = VecDeque::new();
-        TerminalPlotter {
-            width: opt.width,
-            range: opt.range,
-            drawing_area,
-            series,
-        }
-    }
-
     fn update(&mut self, y: f64) {
-        for (x, _) in self.series.iter_mut() {
-            *x = *x - 1.0;
-        }
-        self.series.push_back((0.0, y));
-
-        if self.series.len() > self.width {
-            self.series.pop_front();
-        }
-
-        self.draw_chart().unwrap();
-    }
-}
-
-impl TerminalPlotter {
-    fn draw_chart(&mut self) -> Result<(), Box<dyn Error>> {
-        let drawing_area = self.drawing_area.lock().unwrap();
-
-        clear_screen();
-
-        let x_range = (-(self.width as f64))..0f64;
-        let y_range = self.range.min..self.range.max;
-        let y_label_size = (5i32).percent_width();
-        let x_label_size = (10i32).percent_height();
-
-        let mut chart = ChartBuilder::on(&drawing_area)
-            .margin(1)
-            .set_label_area_size(LabelAreaPosition::Left, y_label_size)
-            .set_label_area_size(LabelAreaPosition::Bottom, x_label_size)
-            .build_cartesian_2d(x_range, y_range)?;
-
-        chart
-            .configure_mesh()
-            .disable_x_mesh()
-            .disable_y_mesh()
-            .draw()?;
-
-        let series = self.series.clone();
-
-        chart.draw_series(LineSeries::new(series.into_iter(), &RED))?;
-
-        drawing_area.present()?;
-
-        Ok(())
+        let record = (Instant::now(), y);
+        let hr = HistoryRecord {
+            record,
+            history_id: self.history_id,
+        };
+        self.tx.send(hr).unwrap();
     }
 }
 
@@ -241,3 +199,132 @@ impl DrawingBackend for TextDrawingBackend {
 fn clear_screen() {
     print!("\x1B[2J\x1B[1;1H");
 }
+
+pub(crate) struct TerminalMultiPlotter {
+    tx: Sender,
+    children_count: usize,
+    _thread: std::thread::JoinHandle<()>,
+}
+
+struct Worker {
+    opt: PlotterOpt,
+    max_elapsed_time: Duration,
+    drawing_area: DrawingArea<TextDrawingBackend, plotters::coord::Shift>,
+    histories: Vec<History>,
+    next_draw_time: Instant,
+}
+
+impl MultiPlotter for TerminalMultiPlotter {
+    fn new(opt: PlotterOpt) -> Self
+    where
+        Self: Sized,
+    {
+        let (tx, mut rx) = tokio::sync::mpsc::unbounded_channel();
+
+        let thread = std::thread::spawn(move || loop {
+            let backend = TextDrawingBackend {
+                state: vec![PixelState::Empty; 5000],
+                width: opt.width,
+            };
+
+            let drawing_area = backend.into_drawing_area();
+
+            let mut worker = Worker {
+                opt,
+                drawing_area,
+                histories: Vec::default(),
+                max_elapsed_time: Duration::from_secs(opt.width as u64),
+                next_draw_time: Instant::now(),
+            };
+
+            while let Some(hr) = rx.blocking_recv() {
+                worker.update_history(hr);
+
+                let now = Instant::now();
+
+                if now.ge(&worker.next_draw_time) {
+                    worker.draw_chart().unwrap();
+                    worker.next_draw_time = now + Duration::from_millis(40);
+                }
+            }
+        });
+
+        TerminalMultiPlotter {
+            tx,
+            children_count: 0,
+            _thread: thread,
+        }
+    }
+
+    fn spawn(&mut self) -> Box<dyn Plotter + Send> {
+        let tx = self.tx.clone();
+        self.children_count += 1;
+        let history_id = self.children_count;
+        let plotter = TerminalPlotter { tx, history_id };
+
+        Box::new(plotter)
+    }
+}
+
+impl Worker {
+    fn draw_chart(&mut self) -> Result<(), Box<dyn Error>> {
+        let drawing_area = &mut self.drawing_area;
+
+        clear_screen();
+
+        let width = self.opt.width;
+        let range = self.opt.range;
+
+        let x_range = (-(width as f64))..0f64;
+        let y_range = range.min..range.max;
+        let y_label_size = (5i32).percent_width();
+        let x_label_size = (10i32).percent_height();
+
+        let mut chart = ChartBuilder::on(drawing_area)
+            .margin(1)
+            .set_label_area_size(LabelAreaPosition::Left, y_label_size)
+            .set_label_area_size(LabelAreaPosition::Bottom, x_label_size)
+            .build_cartesian_2d(x_range, y_range)?;
+
+        chart
+            .configure_mesh()
+            .disable_x_mesh()
+            .disable_y_mesh()
+            .draw()?;
+
+        for history in &self.histories {
+            let history = history
+                .0
+                .iter()
+                .map(|(x, y)| (-x.elapsed().as_secs_f64(), *y));
+            chart.draw_series(LineSeries::new(history, &RED))?;
+        }
+
+        drawing_area.present()?;
+
+        Ok(())
+    }
+
+    fn update_history(&mut self, hr: HistoryRecord) {
+        let HistoryRecord { history_id, record } = hr;
+
+        if history_id >= self.histories.len() {
+            self.histories.push(History::default());
+        }
+
+        let max_elapsed_time = self.max_elapsed_time;
+
+        if let Some(history) = self.histories.get_mut(history_id) {
+            let history = &mut history.0;
+
+            history.push_back(record);
+
+            let to_be_removed = history
+                .iter()
+                .take_while(|(x, _)| x.elapsed() > max_elapsed_time)
+                .count();
+
+            history.drain(0..to_be_removed);
+        }
+    }
+}
