@@ -1,6 +1,7 @@
 use super::Plotter;
 use super::PlotterOpt;
 use super::Range;
+use crate::MultiPlotter;
 
 fn calculate_bar_width(x: f64, min: f64, max: f64, width: usize) -> usize {
     let y = ((x - min) / (max - min)) * (width as f64);
@@ -14,10 +15,6 @@ fn calculate_bar_width(x: f64, min: f64, max: f64, width: usize) -> usize {
 pub type StdoutTextPlotter = TextPlotter<std::io::Stdout>;
 
 impl Plotter for StdoutTextPlotter {
-    fn new(opt: PlotterOpt) -> Self {
-        Self::new(opt, std::io::stdout())
-    }
-
     fn update(&mut self, y: f64) {
         self.update(y);
     }
@@ -46,22 +43,52 @@ impl<Out: std::io::Write> TextPlotter<Out> {
     }
 
     fn to_string(&self, x: f64) -> String {
-        let bar_width = calculate_bar_width(x, self.range.min, self.range.max, self.width);
-        let overflow = bar_width > self.width;
-        let bar = if overflow {
-            // TODO Show overflow icon
-            std::iter::repeat("=").take(self.width).collect::<String>()
+        use std::iter::{once, repeat};
+
+        let above_max = x > self.range.max;
+        let below_min = x < self.range.min;
+
+        let bar = if above_max {
+            once("[")
+                .chain(repeat("#").take(self.width))
+                .chain(once("X"))
+                .collect::<String>()
+        } else if below_min {
+            once("X")
+                .chain(repeat(" ").take(self.width))
+                .chain(once("]"))
+                .collect::<String>()
         } else {
+            let bar_width = calculate_bar_width(x, self.range.min, self.range.max, self.width);
             let padding_width = self.width - bar_width;
-            std::iter::repeat("=")
-                .take(bar_width)
-                .chain(std::iter::repeat(".").take(padding_width))
+            once("[")
+                .chain(repeat("#").take(bar_width))
+                .chain(repeat(" ").take(padding_width))
+                .chain(once("]"))
                 .collect::<String>()
         };
+
         [bar, format!("{}", x)].join(" ")
     }
 }
 
+pub(crate) struct StdoutTextMultiPlotter {
+    opt: PlotterOpt,
+}
+
+impl MultiPlotter for StdoutTextMultiPlotter {
+    fn new(opt: PlotterOpt) -> Self
+    where
+        Self: Sized,
+    {
+        StdoutTextMultiPlotter { opt }
+    }
+
+    fn spawn(&mut self) -> Box<dyn Plotter + Send> {
+        Box::new(StdoutTextPlotter::new(self.opt, std::io::stdout()))
+    }
+}
+
 #[cfg(test)]
 mod tests {
     use super::*;
@@ -81,6 +108,6 @@ mod tests {
         let mut tp = TextPlotter::new(opt, output);
         tp.update(5.0);
         tp.update(2.0);
-        assert_eq!(tp.output, b"=====..... 5\n==........ 2\n");
+        assert_eq!(tp.output, b"[#####     ] 5\n[##        ] 2\n");
     }
 }
